@@ -1,3 +1,4 @@
+# coding:utf-8
 """PooledDB - pooling for DB-API 2 connections.
 
 Implements a pool of steady, thread-safe cached connections
@@ -157,6 +158,7 @@ class PooledDB(Pool_db.PooledDB):
 
     version = __version__
     def_validate_sql = 'SELECT 1 FROM DUAL'
+
     def __init__(
             self, creator, mincached=0, maxcached=0,
             maxshared=0, maxconnections=0, blocking=False,
@@ -242,9 +244,9 @@ class PooledDB(Pool_db.PooledDB):
         self.testOnBorrow = test_on_borrow
         self.testIdle = test_idle
         self.validationSql = self.def_validate_sql if not validation_sql else validation_sql
-        self.idleCheckTime = 0 if not idle_check_time else idle_check_time
-        self.validateTimeOut = 1 if not validate_timeout else validate_timeout
-        self.maxWaitTime = max_wait_time
+        self.idleCheckTime = 0 if int(idle_check_time) < 0 else int(idle_check_time)
+        self.validateTimeOut = 1 if int(validate_timeout) < 0 else int(validate_timeout)
+        self.maxWaitTime = 5 if int(max_wait_time) < 0 else int(max_wait_time)
         # 新增项处理结束
         if mincached is None:
             mincached = 0
@@ -280,6 +282,7 @@ class PooledDB(Pool_db.PooledDB):
         idle = [self.dedicated_connection() for i in range(mincached)]
         while idle:
             idle.pop().close()
+        # 开启空闲线程检查
         if self.testIdle and self.idleCheckTime > 0:
             t = Timer(self.idleCheckTime, self.start_check_idle, [self.idleCheckTime])
             t.start()
@@ -317,7 +320,8 @@ class PooledDB(Pool_db.PooledDB):
                         threading.currentThread().getName(), time.time()-start, t.getName(), t.isAlive(), t.error)
                     raise Pool_db.PooledDBError(error)
             else:
-                conn = super().connection(shareable)
+                conn = self.new_connection(shareable)
+                # conn = super().connection(shareable)
             # self._lock.notify()
         except Exception as e:
             raise Pool_db.InvalidConnection(e)
@@ -330,7 +334,8 @@ class PooledDB(Pool_db.PooledDB):
         conn = None
         try:
             while True:
-                conn = super().connection(shareable)
+                conn = self.new_connection(shareable)
+                # conn = super(self).connection(shareable)
                 if self._validate_connection(conn):
                     break
             return conn
@@ -338,6 +343,69 @@ class PooledDB(Pool_db.PooledDB):
             self.close_conn(conn)
             raise Pool_db.PooledDBError("get_validated_conn error", e)
 
+    def new_connection(self, shareable=True):
+        """
+        鉴于python2中无法通过super调用经典类中被重写的方法将父类中 connection()方法重命名
+        self.new_connection() == super().connection()
+
+        Get a steady, cached DB-API 2 connection from the pool.
+
+        If shareable is set and the underlying DB-API 2 allows it,
+        then the connection may be shared with other threads.
+
+        """
+        if shareable and self._maxshared:
+            self._lock.acquire()
+            try:
+                while (not self._shared_cache and self._maxconnections
+                        and self._connections >= self._maxconnections):
+                    self._wait_lock()
+                if len(self._shared_cache) < self._maxshared:
+                    # shared cache is not full, get a dedicated connection
+                    try:  # first try to get it from the idle cache
+                        con = self._idle_cache.pop(0)
+                    except IndexError:  # else get a fresh connection
+                        con = self.steady_connection()
+                    else:
+                        con._ping_check()  # check this connection
+                    con = Pool_db.SharedDBConnection(con)
+                    self._connections += 1
+                else:  # shared cache full or no more connections allowed
+                    self._shared_cache.sort()  # least shared connection first
+                    con = self._shared_cache.pop(0)  # get it
+                    while con.con._transaction:
+                        # do not share connections which are in a transaction
+                        self._shared_cache.insert(0, con)
+                        self._wait_lock()
+                        self._shared_cache.sort()
+                        con = self._shared_cache.pop(0)
+                    con.con._ping_check()  # check the underlying connection
+                    con.share()  # increase share of this connection
+                # put the connection (back) into the shared cache
+                self._shared_cache.append(con)
+                self._lock.notify()
+            finally:
+                self._lock.release()
+            con = Pool_db.PooledSharedDBConnection(self, con)
+        else:  # try to get a dedicated connection
+            self._lock.acquire()
+            try:
+                while (self._maxconnections
+                        and self._connections >= self._maxconnections):
+                    self._wait_lock()
+                # connection limit not reached, get a dedicated connection
+                try:  # first try to get it from the idle cache
+                    con = self._idle_cache.pop(0)
+                except IndexError:  # else get a fresh connection
+                    con = self.steady_connection()
+                else:
+                    con._ping_check()  # check connection
+                con = Pool_db.PooledDedicatedDBConnection(self, con)
+                self._connections += 1
+            finally:
+                self._lock.release()
+        return con
+
     def close_conn(self, conn):
         if not conn:
             return False
